#ifndef lint
static char *RCSid = "$Header: /usr/src/ecn/getethers/RCS/main.c,v 1.1 91/11/27 10:56:32 davy Exp $";
#endif
/*
 * getethers - get hostname/ethernet address information for all hosts on
 *	       an ethernet.
 *
 * David A. Curry
 * Purdue University
 * Engineering Computer Network
 * davy@ecn.purdue.edu
 * November, 1991
 *
 * $Log:	main.c,v $
 * Revision 1.1  91/11/27  10:56:32  davy
 * Initial revision
 * 
 */
#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <netdb.h>
#include <stdio.h>
#include "defs.h"

char	*pname;
char	*filename = NULL;
int	begin_num = 1;
int	end_num = 255;
int	max_hosts = 255;


char	hl_name[256];
char	hl_inet[64];
char	hl_ether[64];

main(argc, argv)
char **argv;
int argc;
{
	char *p;
	FILE *fp;
	u_long network;
	char *get_arp();
	struct hostent *hp;
	struct in_addr addr;
	char hname[64], fname[BUFSIZ];
	int logging, c, lna, verbose;
	extern char *optarg;
	extern int optind;

	pname = *argv;
	verbose  = 0;

	if (argc < 2)
		usage();

	/*
	 * Get our hostname.
	 */
	if (gethostname(hname, sizeof(hname)) < 0) {
		error("gethostname");
		exit(1);
	}

	/*
	 * Process arguments.
	 */
	while ((c = getopt(argc, argv, "vlb:e:f:")) != EOF)
		switch(c)
		{
		case 'v':			/* verbose	*/
			verbose = 1;
			break;
		case 'b':
			begin_num = atoi(optarg);
			break;
		case 'e':
			end_num = atoi(optarg);
			break;
		case 'f':
			filename = optarg;
			break;
		case 'l':
			logging++;
			break;
		case '?':
			usage();
			break;
		}
	max_hosts = end_num-begin_num+1;
	if (verbose)
		printf("max %d hosts\n", max_hosts);
	if (logging)
		filename = argv[optind];

	if (filename)
	{
		if ((fp = fopen(filename, "w")) == NULL) 
		{
			error(filename);
			exit(1);
		}
	}
		

	/*
	 * Convert the given network address to an internet
	 * address.
	 */
	network = inet_network(argv[optind]);
	addr = inet_makeaddr(network, 0);

	/*
	 * Find the ethernet interface that's on that network.
	if ((lna = check_if(addr, &hosts)) < 0) {
		fprintf(stderr, "%s: this host is not on the %s net.\n",
			pname, inet_ntoa(addr));
		exit(1);
	}
	 */

	/*
	 * Save our hostname.  check_if() filled in our
	 * internet address and ethernet address.
	hosts[lna].hl_name = hname;
	 */

	if (verbose)
		printf("%s:\n    ", argv[optind]);

	/*
	 * For each host...
	 */
	for (lna = begin_num; lna <= end_num; lna++) {
		if ((verbose == 1) && ((lna % 4) == 0)) {
			printf("%d.", lna);
			fflush(stdout);
		}

		/*
		 * Build the internet address.
		 */
		addr = inet_makeaddr(network, lna);

		/*
		 * Ping it, and if it's up...
		 */
		if (ping(addr, lna)) {
			/*
			 * Get the hostname.
			 */
			hp = gethostbyaddr(&addr.s_addr,
					   sizeof(addr.s_addr), AF_INET);

			/*
			 * Save the hostname.
			 */
			if (hp != NULL)
				strcpy(hl_name, hp->h_name);
			else
				strcpy(hl_name, "???");

			/*
			 * Save the internet address and get the
			 * ethernet address from the arp table.
			 */
			strcpy(hl_inet, inet_ntoa(addr));
			strcpy(hl_ether, get_arp(addr));
			if (filename) 
			{
				fprintf(fp, "%s\t%s\t%s\n", hl_name,
					hl_inet, hl_ether);
				fflush(fp);
			}
			printf("%-24s%-16s%-24s\n", hl_name, 
				hl_inet, hl_ether);
		}
		else
		{
			printf("\r%3d\r", lna);
			fflush(stdout);
		}
	}

	if (filename)
		fclose(fp);
	exit(0);
}

/*
 * error - perror with program name.
 */
error(s)
char *s;
{
	fprintf(stderr, "%s: ", pname);
	perror(s);
}

usage()
{
	fprintf(stderr, 
		"Usage: %s [-v] [-b num] [-e num] [-l] [-f file] network\n",
			pname);
	exit(1);
}
